
#FUNCTION TO CALCULATE POSTERIOR PROBS (DC)
calcDC  = function(cond,knownRef,knownRel,verbose=FALSE) { #this function differs in what references are conditoned on
 postProb = list() 
 for(loc in locs) {
#   loc=locs[1]
   postProb[[loc]] = list()
   freq = dat$popFreq[[loc]]
   nU = nC - sum(cond>0) #number of unknown
   refK = unlist(dat$refData[[loc]][unique( c(which(cond>0),knownRef) ) ]) #get alleles of conditional reference(s)
   refR = unlist(dat$refData[[loc]][knownRel]) #get alleles of related reference conditional 
   if(nU>0) {
    Glist = calcGjoint(freq=freq,nU=nU,fst=fst,refK=refK,refR=refR ,ibd=ibd, sortComb=TRUE)
   } else {
    Glist = calcGjoint(freq=freq,nU=1,fst=fst,refK=refK,refR=refR ,ibd=ibd)
   }
   G = Glist$G #genotype outcome
   Gv = paste0(G[,1],"/",G[,2]) #get genotype outcome (vector)

   #Get outcome of alleles and allele tables
   Aset = names(freq)
   nAG = matrix(0,nrow=length(Aset),ncol=nrow(G)) #create allele counting matrix for unknown contributors
   rownames(nAG) = Aset
   condGind = rep(0,sum(cond>0)) #get index of genotypes for refernces
   for(rr in 1:length(rN)) { #for each ref
    if(cond[rr]==0) next #skip if not conditioned on  
    ref <- dat$refData[[loc]][[rr]] #get alleles of ref  
    condGind[cond[rr]] <- which( (ref[1]==G[,1] & ref[2]==G[,2]) | (ref[2]==G[,1] & ref[1]==G[,2]) )    #consider unordered alleles in ref
   }
   for(i in 1:nrow(G)) nAG[,i] = table(factor(G[i,],level=Aset)) #insert contribution matrix for the unknowns

   #SPAN OUT ALL INDICES FOR ALL CONTRIBUTORS
   dimv <- dim(Glist$Gprob) #get dimension
   Gcomb <- list()
   for(k in 1:length(dimv)) Gcomb[[k]] <- 1:dimv[k]
   combGind <- expand.grid(Gcomb) #span out all indices 
   totG = nrow(combGind) #number of joint genotype outcome of unknowns
   if(nU==0) totG = 1

   #PREPARE STUTTER-VARIABLES:
   Aadd = numeric() #Number of "empty" non-observed alleles added
   ASind <- rep(0,length(Aset)) #gives index in Aset which the allele stutter to
   if(is.null(xi0) && length(Aset)>1 ) { #if stutter and if at least two alleles in allele outcome
    Astutt <- as.character(as.numeric(Aset[-length(Aset)])-1)
    Aadd <- Astutt[!Astutt%in%Aset] #alleles to add
    tmp <- rep(0,length(Aadd))
    names(tmp) <- Aadd
    freq <- c(freq,tmp) #add stutt-alles to freq
    Aset <- c(Aset,Aadd) #add stutt-alles to Allele outcome
    tmp <- matrix(0,ncol=ncol(nAG),nrow=length(Aadd))
    rownames(tmp) <- Aadd
    nAG <- rbind(nAG,tmp) #add to number of alleles-outcome matrix
    ASind <- rep(0,length(Aset)) #gives index in Aset which the allele stutter to
    Astutt <- as.character(as.numeric(Aset)-1) #THIS MUST BE UPDATED! 
   # ASind[1:length(Astutt)] <- match(Astutt,Aset) #insert index which they stutter to (this causes bug)
    indS = match(as.character(Astutt),as.character(Aset)) #get stuttering inds.
    ASind[which(!is.na(indS))] <-  na.omit(indS)  #index which they stutter to
   }

   #Prep PH data vector:
   Y = rep(0,length(Aset))
   Y[match(dat$samples[[1]][[loc]]$adata,Aset)] <- dat$samples[[1]][[loc]]$hdata
#   names(Y) = Aset #check that this is CORRECT!

   #Prep params:
   mxvec = th[1:(nC-1)]
   mxvec = c(mxvec,1-sum(mxvec))
   if(nC==1) mxvec = 1
   shape0 = 1/th[nC+1]^2 #1/sigma^2
   scale0 = th[nC]/shape0 #mu*sigma^2
   beta = 1 #default is no degradation
   xi = 0  #default is no stutter
   bpv = rep(1,length(Aset)) #base pair vector
   if(!is.null(kit0)) { #if degradation
    #get bp values from kit
    beta = th[nC+2] 
    kitinfo <- getKit(kit0)
    subkit <- subset(kitinfo,toupper(kitinfo$Marker)==loc) #control on dyer
    for(aa in Aset) { #for each allele
     ind <- which(subkit$Allele==aa)
     if(length(ind)==0) ind <- which.min(abs(as.numeric(subkit$Allele) - as.numeric(aa))) #nearest neighbour (in allele name)-> maximum of the alleles
     bpv[aa==Aset] <- subkit$Size[ind] #insert base pair
    }
   } 
   if(is.null(xi0)) xi = tail(th,1) #get last param
   pE <- pA <- rep(0,totG) #weights to observations
   combGv = rep(NA,totG) #REGISTER COMBINED GENOTYPES (strings)
   for(gg in 1:totG) { #for each outcome
    Gind <- condGind #genotype to condition on
    if(nU>0) {
       x2 <- x <- as.integer(combGind[gg,])
       if(nU>2) x2[-1] <- sort(x2[-1],decreasing=TRUE) #sort indices since we only consider ordered Gind (calcGjoint)
       Gind  = c(Gind,x) #get index of genotype of unknown individuals
       pA[gg] = log(Glist$Gprob[rbind(x2)]) #insert genotype prob (log scale)
    }
    mui <-  c(nAG[,Gind,drop=FALSE]%*%mxvec*shape0) #NB; remember the conditional genotypes
    if(beta!=0) mui  <- mui*exp( (bpv-125)/100*log(beta) )  #in case of degradation
    if(xi>0) { #if stutter is considered
     fromStutt = which(ASind>0) #alleleinds to stutter from
     toStutt = ASind[fromStutt] #alleleinds to stutter to
     stutt1 <- mui[fromStutt]*xi #backward-stutter parts
     mui[fromStutt] <- mui[fromStutt]*(1-xi) #remove stutter part (fromAlleles)
     mui[toStutt] <-  mui[toStutt] + stutt1 #add stutter part (toAlleles)
    }
    psiDI <- which( Y>0 & mui==0 ) #drop-in (not a stutter). ANY DROPIN?
    psiYmu <- which( Y>0 & mui>0 )  #contributing to model and observed PH
    psiDO <- which( Y==0 & mui>0 )  #dropout elem

    vali = 0
    if(length(psiYmu)>0) vali =  vali + sum(dgamma(Y[psiYmu],shape=mui[psiYmu],scale=scale0,log=TRUE))   # CONTRIBUTION TO PH
    if(length(psiDO)>0) vali = vali + sum(pgamma(threshT,shape=mui[psiDO],scale=scale0,log=TRUE)) #CONTR TO DROPOUT
    if(length(psiDI)>0) vali = vali + sum( dexp( Y[psiDI]-threshT, rate=lambda, log=TRUE) + log(pC*freq[psiDI]) ) #CONTR TO DROPIN: Scaled for each dropin. THIS IS OK
    if(length(psiDI)==0) vali = vali + log(1-pC) #in case of no dropin
    pE[gg] = vali
    combGv[gg] = paste0(Gv[Gind],collapse=",")
   } #end for each iter

   #CALCULATE JOINT PROBS
   C <- log(sum(exp(pE + pA))) #insert logLik for marker
   jProb = exp(pE + pA - C)  
   ord = order(jProb,decreasing=TRUE)
   jtab  <- data.frame(Genotypes = combGv[ord], Probability = jProb[ord],stringsAsFactors=FALSE)
   #sum(as.numeric(jtab[,2]))

   #CALCULATE MARGINAL (genotypes) for each contr
   Gmat = matrix(unlist(strsplit(jtab[,1],",")),ncol=nrow(jtab))
   mGtab <- mAtab <- list() #for each contributor
   for(cc in 1:nrow(Gmat)) { #for each contributors
    agg = aggregate(jtab[,2],by=list(Gmat[cc,]),sum) #aggregate similar genotypes
    colnames(agg) = c("Genotype","Probability")
    agg = agg[order(agg[,2],decreasing=TRUE),] 
    mGtab[[paste0("C",cc)]] = agg
   }

   #CALCULATE MARGINAL (allele) for each contr
   for(cc in 1:length(mGtab)) { #for each contributors
    Amat = matrix(unlist(strsplit(mGtab[[cc]][,1],"/")),ncol=nrow(mGtab[[cc]]))
    unA = unique(c(Amat)) #get unique alleles
    probA = rep(NA,length(unA))
    for(aa in unA) {
      probA[which(unA==aa)] = sum( as.numeric(colSums(Amat==aa)>0)*mGtab[[cc]][,2] )
    }
    #sum(probA)-sum(mGtab[[cc]][Amat[1,]!=Amat[2,],2]) #outcome prob should be one
    ord = order(probA,decreasing=TRUE)
    mAtab[[paste0("C",cc)]] = data.frame(Allele = unA[ord], Probability = probA[ord],stringsAsFactors=FALSE)
   }
   postProb[[loc]] = list(joint=jtab,margG=mGtab,margA=mAtab)
  } #end for each loci
  return(postProb)
} #end function

#FUNCTION TO GET logLik values for quantiative model
calcLogLikQUAN = function(cond,knownRef,knownRel,verbose=FALSE) { #this function differs in what references are conditoned on
 loglikvals = rep(NA,length(locs))
 for(loc in locs) {
#   loc=locs[2]
   freq = dat$popFreq[[loc]]
   nU = nC - sum(cond>0) #number of unknown
   refK = unlist(dat$refData[[loc]][unique( c(which(cond>0),knownRef) ) ]) #get alleles of conditional reference(s)
   refR = unlist(dat$refData[[loc]][knownRel]) #get alleles of related reference conditional 
   if(nU>0) {
    Glist = calcGjoint(freq=freq,nU=nU,fst=fst,refK=refK,refR=refR ,ibd=ibd, sortComb=TRUE)
   } else {
    Glist = calcGjoint(freq=freq,nU=1,fst=fst,refK=refK,refR=refR ,ibd=ibd)
   }
   G = Glist$G #genotype outcome

   #Get outcome of alleles and allele tables
   Aset = names(freq)
   nAG = matrix(0,nrow=length(Aset),ncol=nrow(G)) #create allele counting matrix for unknown contributors
   rownames(nAG) = Aset
   condGind = rep(0,sum(cond>0)) #get index of genotypes for refernces
   for(rr in 1:length(rN)) { #for each ref
    if(cond[rr]==0) next #skip if not conditioned on  
    ref <- dat$refData[[loc]][[rr]] #get alleles of ref  
    condGind[cond[rr]] <- which( (ref[1]==G[,1] & ref[2]==G[,2]) | (ref[2]==G[,1] & ref[1]==G[,2]) )    #consider unordered alleles in ref
   }
   for(i in 1:nrow(G)) nAG[,i] = table(factor(G[i,],level=Aset)) #insert contribution matrix for the unknowns

   #SPAN OUT ALL INDICES FOR ALL CONTRIBUTORS
   dimv <- dim(Glist$Gprob) #get dimension
   Gcomb <- list()
   for(k in 1:length(dimv)) Gcomb[[k]] <- 1:dimv[k]
   combGind <- expand.grid(Gcomb) #span out all indices 
   totG = nrow(combGind) #number of joint genotype outcome of unknowns
   if(nU==0) totG = 1

   #PREPARE STUTTER-VARIABLES:
   Aadd = numeric() #Number of "empty" non-observed alleles added
   ASind <- rep(0,length(Aset)) #gives index in Aset which the allele stutter to
   if(is.null(xi0) && length(Aset)>1 ) { #if stutter and if at least two alleles in allele outcome
    Astutt <- as.character(as.numeric(Aset[-length(Aset)])-1)
    Aadd <- Astutt[!Astutt%in%Aset] #alleles to add
    tmp <- rep(0,length(Aadd))
    names(tmp) <- Aadd
    freq <- c(freq,tmp) #add stutt-alles to freq
    Aset <- c(Aset,Aadd) #add stutt-alles to Allele outcome
    tmp <- matrix(0,ncol=ncol(nAG),nrow=length(Aadd))
    rownames(tmp) <- Aadd
    nAG <- rbind(nAG,tmp) #add to number of alleles-outcome matrix
    ASind <- rep(0,length(Aset)) #gives index in Aset which the allele stutter to
    Astutt <- as.character(as.numeric(Aset)-1) #THIS MUST BE UPDATED! 
   # ASind[1:length(Astutt)] <- match(Astutt,Aset) #insert index which they stutter to (this causes bug)
    indS = match(as.character(Astutt),as.character(Aset)) #get stuttering inds.
    ASind[which(!is.na(indS))] <-  na.omit(indS)  #index which they stutter to
    }

   #Prep PH data vector:
   Y = rep(0,length(Aset))
   Y[match(dat$samples[[1]][[loc]]$adata,Aset)] <- dat$samples[[1]][[loc]]$hdata
#   names(Y) = Aset #check that this is CORRECT!

   #Prep params:
   mxvec = th[1:(nC-1)]
   mxvec = c(mxvec,1-sum(mxvec))
   if(nC==1) mxvec = 1
   shape0 = 1/th[nC+1]^2 #1/sigma^2
   scale0 = th[nC]/shape0 #mu*sigma^2
   beta = 1 #default is no degradation
   xi = 0  #default is no stutter
   bpv = rep(1,length(Aset)) #base pair vector
   if(!is.null(kit0)) { #if degradation
    #get bp values from kit
    beta = th[nC+2] 
    kitinfo <- getKit(kit0)
    subkit <- subset(kitinfo,toupper(kitinfo$Marker)==loc) #control on dyer
    for(aa in Aset) { #for each allele
     ind <- which(subkit$Allele==aa)
     if(length(ind)==0) ind <- which.min(abs(as.numeric(subkit$Allele) - as.numeric(aa))) #nearest neighbour (in allele name)-> maximum of the alleles
     bpv[aa==Aset] <- subkit$Size[ind] #insert base pair
    }
   } 
   if(is.null(xi0)) xi = tail(th,1) #get last param
   pE <- pA <- rep(0,totG) #weights to observations
   for(gg in 1:totG) { #for each outcome
    Gind <- condGind #genotype to condition on
    if(nU>0) {
       x2 <- x <- as.integer(combGind[gg,])
       if(nU>2) x2[-1] <- sort(x2[-1],decreasing=TRUE) #sort indices since we only consider ordered Gind (calcGjoint)
       Gind  = c(Gind,x) #get index of genotype of unknown individuals
       pA[gg] = log(Glist$Gprob[rbind(x2)]) #insert genotype prob (log scale)
    }
    mui <-  c(nAG[,Gind,drop=FALSE]%*%mxvec*shape0) #NB; remember the conditional genotypes
    if(beta!=0) mui  <- mui*exp( (bpv-125)/100*log(beta) )  #in case of degradation
    if(xi>0) { #if stutter is considered
     fromStutt = which(ASind>0) #alleleinds to stutter from
     toStutt = ASind[fromStutt] #alleleinds to stutter to
     stutt1 <- mui[fromStutt]*xi #backward-stutter parts
     mui[fromStutt] <- mui[fromStutt]*(1-xi) #remove stutter part (fromAlleles)
     mui[toStutt] <-  mui[toStutt] + stutt1 #add stutter part (toAlleles)
    }
    psiDI <- which( Y>0 & mui==0 ) #drop-in (not a stutter). ANY DROPIN?
    psiYmu <- which( Y>0 & mui>0 )  #contributing to model and observed PH
    psiDO <- which( Y==0 & mui>0 )  #dropout elem

    vali = 0
    if(length(psiYmu)>0) vali =  vali + sum(dgamma(Y[psiYmu],shape=mui[psiYmu],scale=scale0,log=TRUE))   # CONTRIBUTION TO PH
    if(length(psiDO)>0) vali = vali + sum(pgamma(threshT,shape=mui[psiDO],scale=scale0,log=TRUE)) #CONTR TO DROPOUT
    if(length(psiDI)>0) vali = vali + sum( dexp( Y[psiDI]-threshT, rate=lambda, log=TRUE) + log(pC*freq[psiDI]) ) #CONTR TO DROPIN: Scaled for each dropin. THIS IS OK
    if(length(psiDI)==0) vali = vali + log(1-pC) #in case of no dropin
    pE[gg] = vali

   } #end for each iter
   loglikvals[which(loc==locs)] <- log(sum(exp(pE + pA))) #insert logLik for marker
  } #end for each loci
  if(verbose) print( loglikvals ) #print to screen
  return(sum(loglikvals))
} #end function


#FUNCTION TO GET logLik values
calcLogLikQUAL = function(cond,knownRef,knownRel,verbose=FALSE) { #this function differs in what references are conditoned on
 loglikvals = rep(NA,length(locs))
 for(loc in locs) {
#   loc=locs[13]
   freq = dat$popFreq[[loc]]
   nU = nC - sum(cond>0) #number of unknown
   refK = unlist(dat$refData[[loc]][unique( c(which(cond>0),knownRef) ) ]) #get alleles of conditional reference(s)
   refR = unlist(dat$refData[[loc]][knownRel]) #get alleles of related reference conditional 
   if(nU>0) {
    Glist = calcGjoint(freq=freq,nU=nU,fst=fst,refK=refK,refR=refR ,ibd=ibd, sortComb=TRUE)
   } else {
    Glist = calcGjoint(freq=freq,nU=1,fst=fst,refK=refK,refR=refR ,ibd=ibd)
   }
   G = Glist$G #genotype outcome

   #Get outcome of alleles and allele tables
   Aset = names(freq)
   nAG = matrix(0,nrow=length(Aset),ncol=nrow(G)) #create allele counting matrix for unknown contributors
   rownames(nAG) = Aset
   condGind = rep(0,sum(cond>0)) #get index of genotypes for refernces
   for(rr in 1:length(rN)) { #for each ref
    if(cond[rr]==0) next #skip if not conditioned on  
    ref <- dat$refData[[loc]][[rr]] #get alleles of ref  
    condGind[cond[rr]] <- which( (ref[1]==G[,1] & ref[2]==G[,2]) | (ref[2]==G[,1] & ref[1]==G[,2]) )    #consider unordered alleles in ref
   }
   for(i in 1:nrow(G)) nAG[,i] = table(factor(G[i,],level=Aset)) #insert contribution matrix for the unknowns

   #SPAN OUT ALL INDICES FOR ALL CONTRIBUTORS
   dimv <- dim(Glist$Gprob) #get dimension
   Gcomb <- list()
   for(k in 1:length(dimv)) Gcomb[[k]] <- 1:dimv[k]
   combGind <- expand.grid(Gcomb) #span out all indices 
   totG = nrow(combGind) #number of joint genotype outcome of unknowns
   if(nU==0) totG = 1

   #Prep PH data vector:
   Y = rep(0,length(Aset))
   Y[match(dat$samples[[1]][[loc]]$adata,Aset)] <- 1 #include binary

   #Prep params:
   pE <- pA <- rep(0,totG) #weights to observations
   for(gg in 1:totG) { #for each outcome
    Gind <- condGind #genotype to condition on
    if(nU>0) {
       x2 <- x <- as.integer(combGind[gg,])
       if(nU>2) x2[-1] <- sort(x2[-1],decreasing=TRUE) #sort indices since we only consider ordered Gind (calcGjoint)
       Gind  = c(Gind,x) #get index of genotype of unknown individuals
       pA[gg] = log(Glist$Gprob[rbind(x2)]) #insert genotype prob (log scale)
    }
    Xij <-  nAG[,Gind,drop=FALSE] #Contribution
    mui <-  rowSums(Xij) 

    psiDI <- which( Y>0 & mui==0 ) #drop-in (not a stutter). ANY DROPIN?
    psiYmu <- which( Y>0 & mui>0 )  #contributing to model and observed PH
    psiDO <- which( Y==0 & mui>0 )  #dropout elem

    #sumM operates on log-scale (avoids powering)
    sumM = t(t(Xij)*log(pD)) #get contributions
    sumM[is.nan(sumM)] = 0 #insert 0 if no contribution (on log scale)

    vali = 0
    if(length(psiYmu)>0) { #IF CONTRIBUTION TO MODEL (CONSIDER NON-DROPOUT CASES
        val2 <- 1 - exp( rowSums(sumM[psiYmu,,drop=F])) #consider temporary variable
        vali = vali + sum( log( val2 ))
    }
    if(length(psiDO)>0) {   #IF DROPOUT
        vali = vali + sum(sumM[psiDO,]) #add to likelihood
        if(is.nan(vali)) vali <- -Inf #Lik=0 if model not possible (dropout only explanation)
     }        
    if(length(psiDI)>0) vali = vali + sum(log(pC*freq[psiDI]) ) #CONTR TO DROPIN: Scaled for each dropin. THIS IS OK
    if(length(psiDI)==0) vali  = vali + log(1-pC) #in case of no dropin
    pE[gg] = vali
   } #end for each iter
   loglikvals[which(loc==locs)] <- log(sum(exp(pE + pA))) #insert logLik for marker
  } #end for each loci
  if(verbose) print( loglikvals ) #print to screen
  return(sum(loglikvals))
} #end function


 #Function to calculate the MLE based LR (qual) using forensim::likEvid
  qualLRmle = function(pD0=0.1) {  
    getConds <- function(condOrder,knownref) {  #helpfunction to get conditional refs under a hypothesis
      cond <- which(condOrder>0) #ind of conditional refs (they are increasingly sorted)
      Ri <- Ki <- NULL
      for(rr in cond ) Ri <- c(Ri,dat$refData[[loc]][[rr]])
      for(rr in knownref) Ki <- c(Ki,dat$refData[[loc]][[rr]])
      return(list(Ri=Ri,Ki=Ki,nR=length(Ri)/2 ))  #Updated in v1.9: consider number of alleles divided by 2
    }
 	 #PREPARE DATA FOR FORENSIM:
	  nS = length(dat$samples) #number of reps	
	  Evidlist <- list()
	  refList_hp <- list()
	  refList_hd <- list()
	  for(loc in locs) {
	  Ei <- NULL #get evidence
	  for(ss in 1:nS) {
	     if(ss>1) Ei <- c(Ei,0) #seperate with 0  
	     adata <- dat$samples[[ss]][[loc]]$adata
	     if(length(adata)==0) adata=0 #is empty
	     Ei <- c(Ei,adata)
    	  }	 
 	   Evidlist[[loc]] <- Ei
 	   refList_hp[[loc]] <- getConds(condOrder=hpcond,knownref=NULL) #under hp
  	   refList_hd[[loc]] <- getConds(condOrder=hdcond,knownRefHd) #under hd
 	   if(length(refList_hp[[loc]]$Ri)==0) refList_hp[[loc]]$Ri <- NULL 
    	   if(length(refList_hd[[loc]]$Ri)==0) refList_hd[[loc]]$Ri <- NULL
    }
    neglikhp <- function(pD) {
      pDhp <- rep(1/(1+exp(-pD)),nC)
      hpvec <- rep(1,length(locs))
      for(loc in locs) hpvec[which(loc==locs)] <- forensim::likEvid(Repliste=Evidlist[[loc]],T=refList_hp[[loc]]$Ri,V=refList_hp[[loc]]$Ki,x=nC-refList_hp[[loc]]$nR,theta=fst, prDHet=pDhp, prDHom=pDhp^2, prC=pC, freq=dat$popFreq[[loc]])
      return( -sum(log(hpvec)) )
    }

    neglikhd <- function(pD) {
       pDhd <-  rep(1/(1+exp(-pD)),nC)
       hdvec <- rep(1,length(locs))
       for(loc in locs) hdvec[which(loc==locs)] <- forensim::likEvid(Repliste=Evidlist[[loc]],T=refList_hd[[loc]]$Ri,V=refList_hd[[loc]]$Ki,x=nC-refList_hd[[loc]]$nR,theta=fst, prDHet=pDhd, prDHom=pDhd^2, prC=pC, freq=dat$popFreq[[loc]])
       return( -sum(log(hdvec)) )
    }
	pDv = c(0.1,0.35,0.7) #Necessary to look for potential better start point than (0.1) to make optimizer more robust
	loghp = Vectorize(neglikhp)( log(pDv/(1-pDv)) )
    pD0 = pDv[which.min(loghp)] 
    foohp <- nlm(Vectorize(neglikhp),log(pD0/(1-pD0)) )
	
    loghd = Vectorize(neglikhd)( log(pDv/(1-pDv)) )
	pD0 = pDv[which.min(loghd)] 
    foohd <- nlm(Vectorize(neglikhd),log(pD0/(1-pD0)) )

    pDhat <- 1/(1+exp(-c(foohp$est,foohd$est))) #get estimated dropouts
    logLikHp = -foohp$min
    logLikHd = -foohd$min
    log10LRmle =  (logLikHp - logLikHd)/log(10) #get LR on log10 scale
    return( list(LR=exp(logLikHp-logLikHd),log10LR=log10LRmle,hp=list(DO=pDhat[1],logLik=logLikHp),hd=list(DO=pDhat[2],logLik=logLikHd)) )
  } #end qualLRmleBased

####################
#OTHER HELP METHODS#
####################
sample_tableToList = function(X,threshT=NULL) {
  cn = colnames(X) #colnames 
  lind = grep("marker",tolower(cn),fixed=TRUE) #locus col-ind
  if(length(lind)==0) lind = grep("loc",tolower(cn),fixed=TRUE) #try another name
  sind = grep("sample",tolower(cn),fixed=TRUE) #sample col-ind
  if(length(sind)>1)  sind = sind[grep("name",tolower(cn[sind]),fixed=TRUE)] #use only sample name
  A_ind = grep("allele",tolower(cn),fixed=TRUE) #allele col-ind
  H_ind = grep("height",tolower(cn),fixed=TRUE) #height col-ind
  ln = unique(toupper(X[,lind])) #locus names: Convert to upper case
  sn = unique(as.character(X[,sind])) #sample names
  I = length(ln)
  Y = list() #insert non-empty characters:
  for(k in 1:length(sn)) { #for each sample in matrix
   Y[[sn[k]]] = list() #one list for each sample
   for(i in 1:I) { #for each locus
     xind = X[,sind]==sn[k] & toupper(X[,lind])==ln[i] #get index in X for given sample and locus
     if(sum(xind)==0) next
     keep <- which(!is.na(X[xind,A_ind]) & X[xind,A_ind]!="")
     if(length(H_ind)>0) { #If peak heights are considered
      PH <- as.numeric(as.character(X[xind,H_ind][keep])) #get the peak heights
      if(!is.null(threshT)) keep = which(PH>=threshT) #keep only alleles above thrshold (if given)
      Y[[sn[k]]][[ln[i]]]$hdata = PH[keep]
     }
     if(length(A_ind)>0) {
      Y[[sn[k]]][[ln[i]]]$adata = as.character(X[xind,A_ind][keep])
     }
   }   
  }
  names(Y) <- sn
  return(Y)
}

getData <- function(mixData2,refData2,popFreq) { #Helpfunction to get data to analyse
 locs <- names(popFreq)
 mixData <- lapply(mixData2,function(x) return(x[locs])) #return selected loci
 refData <- list()
 for(loc in locs)  refData[[loc]] <- lapply(refData2,function(x) return(x[[loc]]$adata)) #return selected loci
 Qret <- Qassignate(samples=mixData, popFreq,  refData,incS=FALSE,incR=FALSE)
 return(list(samples=mixData,refData=Qret$refData,popFreq=Qret$popFreq))
}
