#setwd("SET DIRECTORY TO THE FILES HERE") #IMPORTANT TO SET WORKDIRECTORY 
require(euroformix); sessionInfo() 
#source(paste0("runDCTests.R"))
source("helpfunctions.R")

evids = c("01.01") #evidence profiles to consider
for(evidN in evids) { #run through each evid profile
print(evidN)

#Model setup:
kit = "GlobalFiler"
threshT = 70 #detection threshold (rfu)
fst = 0
pC = 0.05 #dropin frequency.
lambda = 0.01 #dropin peak height param 

evidFile <- paste0("evid_",evidN,".csv") #Selected evidence file

#Get evidence:
mixData = sample_tableToList( X=tableReader( evidFile),threshT=threshT ) #get sample to analyse. NOTICE THAT THE PEAK HEIGHT THRESHOLD IS GIVEN AS ARGUMENT

#Outfile to store results
outf <- paste0("DC_Results_",evidN, ".txt") #

#get hypothesis file:
hyps = tableReader( paste0("HYP_",evidN,".txt") ) #get hypothesis to consider

#get popfreq file:
databaseFile = "freqs.csv" #The allele frequency file
popFreq <- freqImport(databaseFile)[[1]] #import population freqs.
locs = names(popFreq) #loci to consider
locs = intersect(locs,names(mixData[[1]]))

#get references:
refFile <- "refs.csv" #the file including references
refData=sample_tableToList(tableReader(refFile)) 
rN <- names(refData) #names of references

#plotEPG(Data=mixData,kitname=kit,threshT=threshT,refcond=refData,showPH=TRUE) #plotting evidence with ref
dat <- getData(mixData,refData,popFreq[locs]) #process data for euroformix calculations

#optimization iterations
nDone=4 #Number of required converged estimates 

# out object
out = NULL

# Loop over hypothesis
for(hh in 1:nrow(hyps)) { 
  print( paste0("Comparing hypothesis set ",hh))
  hyp <- hyps[hh,] #get selected hypothesis 
  nC = as.numeric(hyp[1]) #get assumed number of contributors
  xi0 <- kit0 <- NULL 
  if(hyp[4]=="Y")  kit0 = kit #set kit
  if(hyp[5]=="N")  xi0 = 0 #set no stutter
  hpcond <- hdcond <- rep(0,length(rN))
  hpcond[rN%in%hyp[2]] <- 1 #set POI as 1.contr.
  knownRefHd <- which(rN%in%hyp[2]) #condition under Hd that ref i is a known non-contributors. This is a vector specifying which of the i-th references that are known non-contributors under hd.
  if(!is.na(hyp[3]) & hyp[3]!="") {
    hpcond[rN%in%hyp[3]] <- 2 #cond ref is C2 und Hp
    hdcond[rN%in%hyp[3]] <- 1 #cond ref is C1 und Hd
  }
  knownRel = which(rN%in%hyp[6]) 

  #FIT MLE MODEL
  set.seed(3)
  hpfit <- contLikMLE(nC,dat$samples,popFreq=dat$popFreq,threshT=threshT,nDone=nDone,xi=xi0,refData=dat$refData,prC=pC,lambda=lambda,fst=fst,condOrder=hpcond,kit=kit0)
  th = hpfit$fit$thetahat #get estimate theta. Used in function calcLogLik
  
  #PERFORM DC under Hp (euroformix::deconvolve and own implemenation calcDC)
  dc_hp = deconvolve(hpfit)
  dc_hpALT <- calcDC(cond=hpcond,knownRef=NULL,knownRel=NULL) #CHECK WITH ALTERNATIVE METHOD:

  #FIT MLE MODEL under Hd
  set.seed(3)
  hdfit <- contLikMLE(nC,dat$samples,popFreq=dat$popFreq,threshT=threshT,nDone=nDone,xi=xi0,refData=dat$refData,prC=pC,lambda=lambda,fst=fst,condOrder=hdcond,knownRef=knownRefHd,kit=kit0, knownRel = knownRel)
  th = hdfit$fit$thetahat #get estimate theta. Used in function calcLogLik

  #PERFORM DC under Hd (euroformix::deconvolve and own implemenation calcDC)
  dc_hd = deconvolve(hdfit)
  dc_hdALT <- calcDC(cond=hdcond,knownRef=knownRefHd ,knownRel=NULL) #CHECK WITH ALTERNATIVE METHOD:

  #COMPARE PROBABILITIES:
 for(ht in c("hp","hd")) {
  if(ht=="hp") {
   dc_h = dc_hp
   dc_hALT = dc_hpALT
  }
  if(ht=="hd") {
   dc_h = dc_hd
   dc_hALT = dc_hdALT
  }

 errJoint <- errMargG <-  errMargA <- rep(NA,length(locs))
 for(loc in locs) {
  ncol = ncol(dc_h$rankGi[[loc]])
  x1 = as.numeric(dc_h$rankGi[[loc]][,ncol])
  x2 = dc_hALT[[loc]]$joint[,2]
  errJoint[which(locs==loc)] = sum(abs(x1-x2)) #get error for joint

  #Compare top ranked margG:
  x1 = as.numeric(dc_h$toprankGi[[loc]][2,])
  x2 = sapply(dc_hALT[[loc]]$margG,function(x) x[1,2]) #get top
  errMargG[which(locs==loc)] = sum(abs(x1-x2)) #get error for joint

  #Compare top ranked margA:
  indloc = dc_h$table4[,2]==loc
  induse = !duplicated(dc_h$table4[indloc,1])
  x1 = as.numeric(dc_h$table4[which(indloc)[induse],4]) #extract top probs
  x2 = sapply(dc_hALT[[loc]]$margA,function(x) x[1,2]) #get top
  errMargA[which(locs==loc)] = sum(abs(x1-x2)) #get error for joint
 } #end for each loci

  mod = paste0(hyp,collapse="/")
  DC <- c(ht,mod,sum(errJoint),sum(errMargG),sum(errMargA))

  out <-  rbind(out,DC)
 }
 out <-  rbind(out,"----------")
 colnames(out ) <- c("Hyp","Mod","SUM Abs.Err.AllJointProbs","SUM Abs.Err.TopGprobs","SUM Abs.Err.TopAprobs")
  write.table(out,outf,sep="\t")

} #end loop for each hypothesis

} #end loop for each evidence

