#setwd("SET DIRECTORY TO THE FILES HERE") #IMPORTANT TO SET WORKDIRECTORY 
require(euroformix); sessionInfo() 
#source("runQualTests.R")
source("helpfunctions.R")

evids = c("38.06","19.01","40.03") #evidence profiles to consider
for(evidN in evids) { #run through each evid profile

print(evidN)

#Model setup:
kit = "GlobalFiler" #NOT USED FOR QUAL MODEL
threshT = 70 #detection threshold (rfu)
fst = 0
pC = 0.05 #dropin frequency.

evidFile <- paste0("evid_",evidN,".csv") #Selected evidence file

#Get evidence:
mixData = sample_tableToList( X=tableReader( evidFile),threshT=threshT ) #get sample to analyse. NOTICE THAT THE PEAK HEIGHT THRESHOLD IS GIVEN AS ARGUMENT

#Outfile to store results
outf <- paste0("QualResults_",evidN, ".txt") #

#get hypothesis file:
hyps = tableReader( paste0("HYP2_",evidN,".txt") ) #get hypothesis to consider

#get popfreq file:
databaseFile = "freqs.csv" #The allele frequency file
popFreq <- freqImport(databaseFile)[[1]] #import population freqs.
locs = names(popFreq) #loci to consider
locs = intersect(locs,names(mixData[[1]]))

#get references:
refFile <- "refs.csv" #the file including references
refData=sample_tableToList(tableReader(refFile)) 
rN <- names(refData) #names of references

#plotEPG(Data=mixData,kitname=kit,threshT=threshT,refcond=refData,showPH=TRUE) #plotting evidence with ref
dat <- getData(mixData,refData,popFreq[locs]) #process data for euroformix calculations

# out object
out = NULL

for(hh in 1:nrow(hyps)) {  # Loop over hypotheses
  print( paste0("Comparing hypothesis set ",hh))
  hyp <- hyps[hh,] #get selected hypothesis 
  nC = as.numeric(hyp[1]) #get assumed number of contributors
  hpcond <- hdcond <- rep(0,length(rN))
  hpcond[rN%in%hyp[2]] <- 1 #set POI as 1.contr.
  knownRefHd <- which(rN%in%hyp[2]) #condition under Hd that ref i is a known non-contributors. This is a vector specifying which of the i-th references that are known non-contributors under hd.
  if(!is.na(hyp[3]) & hyp[3]!="") {
    hpcond[rN%in%hyp[3]] <- 2 #cond ref is C2 und Hp
    hdcond[rN%in%hyp[3]] <- 1 #cond ref is C1 und Hd
  }
  knownRel = which(rN%in%hyp[4]) 

  #get optimized values by using forensim::likEvid
  opt = qualLRmle() 
  LR1 = opt$LR


  #Consider own implemenation
  pD = opt$hp$DO
  likhp <- calcLogLikQUAL(cond=hpcond,knownRef=NULL,knownRel=NULL)

  pD = opt$hd$DO
  likhd <- calcLogLikQUAL(cond=hdcond,knownRef=knownRefHd,knownRel=NULL)
  LR2 = exp(likhp-likhd)
  log10LR2 = (likhp-likhd)/log(10)

  #Store results to file
  hyp2 = paste0(hyp,collapse="/")
  FORENSIM_C <- paste0(c(hyp2,opt$hd$logLik,opt$hp$logLik,opt$LR,opt$log10LR))
  FORENSIM_R <- paste0(c(hyp2,likhd,likhp,LR2,log10LR2))
  tab <-  rbind(FORENSIM_C,FORENSIM_R,"----------")
  out <- rbind( out , tab )
  colnames(out ) <- c("Hyp","logLikHd","logLikHp","LR","log10LR")

  write.table(out,outf,sep="\t")

} #end loop for each hypothesis

} #end loop for each evidence