#setwd("SET DIRECTORY TO THE FILES HERE") #IMPORTANT TO SET WORKDIRECTORY 
require(euroformix); sessionInfo() 
#source("runQuanTests.R")
source("helpfunctions.R")

evids = c("38.06","40.03") #evidence profiles to consider
for(evidN in evids) { #run through each evid profile

print(evidN)


#Model setup:
kit = "GlobalFiler"
threshT = 70 #detection threshold (rfu)
fst = 0
pC = 0.05 #dropin frequency.
lambda = 0.01 #dropin peak height param 


evidFile <- paste0("evid_",evidN,".csv") #Selected evidence file

#Get evidence:
mixData = sample_tableToList( X=tableReader( evidFile),threshT=threshT ) #get sample to analyse. NOTICE THAT THE PEAK HEIGHT THRESHOLD IS GIVEN AS ARGUMENT

#Outfile to store results
outf <- paste0("QuanResults_",evidN, ".txt") #

#get hypothesis file:
hyps = tableReader( paste0("HYP_",evidN,".txt") ) #get hypothesis to consider

#get popfreq file:
databaseFile = "freqs.csv" #The allele frequency file
popFreq <- freqImport(databaseFile)[[1]] #import population freqs.
locs = names(popFreq) #loci to consider
locs = intersect(locs,names(mixData[[1]]))

#get references:
refFile <- "refs.csv" #the file including references
refData=sample_tableToList(tableReader(refFile)) 
rN <- names(refData) #names of references

#plotEPG(Data=mixData,kitname=kit,threshT=threshT,refcond=refData,showPH=TRUE) #plotting evidence with ref
dat <- getData(mixData,refData,popFreq[locs]) #process data for euroformix calculations

#optimization iterations
nDone=4 #Number of required converged estimates 

# out object
out = NULL

# Loop over hypothesis
for(hh in 1:nrow(hyps)) { 
# hh=1
  print( paste0("Comparing hypothesis set ",hh))
  hyp <- hyps[hh,] #get selected hypothesis 
  nC = as.numeric(hyp[1]) #get assumed number of contributors
  xi0 <- kit0 <- NULL 
  if(hyp[4]=="Y")  kit0 = kit #set kit
  if(hyp[5]=="N")  xi0 = 0 #set no stutter
  hpcond <- hdcond <- rep(0,length(rN))
  hpcond[rN%in%hyp[2]] <- 1 #set POI as 1.contr.
  knownRefHd <- which(rN%in%hyp[2]) #condition under Hd that ref i is a known non-contributors. This is a vector specifying which of the i-th references that are known non-contributors under hd.
  if(!is.na(hyp[3]) & hyp[3]!="") {
    hpcond[rN%in%hyp[3]] <- 2 #cond ref is C2 und Hp
    hdcond[rN%in%hyp[3]] <- 1 #cond ref is C1 und Hd
  }
  knownRel = which(rN%in%hyp[6]) 

  #calculate loglik (mle based) by using euroformix::contLikMLE
  set.seed(3)
  hpfit <- contLikMLE(nC,dat$samples,popFreq=dat$popFreq,threshT=threshT,nDone=nDone,xi=xi0,refData=dat$refData,prC=pC,lambda=lambda,fst=fst,condOrder=hpcond,kit=kit0)
  th = hpfit$fit$thetahat #get estimate theta. Used in function calcLogLik
  hpfit$fit$loglik #get loglik value

  #Consider own implemenation
  likhp <- calcLogLikQUAN(cond=hpcond,knownRef=NULL,knownRel=NULL)

  #calculate loglik (mle based) by using euroformix::contLikMLE
  set.seed(3)
  hdfit <- contLikMLE(nC,dat$samples,popFreq=dat$popFreq,threshT=threshT,nDone=nDone,xi=xi0,refData=dat$refData,prC=pC,lambda=lambda,fst=fst,condOrder=hdcond,knownRef=knownRefHd,kit=kit0, knownRel = knownRel)
  th = hdfit$fit$thetahat #get estimate theta. Used in function calcLogLik
  hdfit$fit$loglik #get loglik value

  #Consider own implemenation
  likhd <- calcLogLikQUAN(cond=hdcond,knownRef=knownRefHd,knownRel=knownRel)

  #Store results to file
  LR1 <- exp(hpfit$fit$loglik - hdfit$fit$loglik)
  LR2 <- exp(likhp - likhd )
 
  hyp2 = paste0(hyp,collapse="/")
  EFM_C <- paste0(c(hyp2,hdfit$fit$loglik,hpfit$fit$loglik,LR1,log10(LR1)))
  EFM_R <- paste0(c(hyp2,likhd,likhp,LR2,log10(LR2)))
  tab <-  rbind(EFM_C,EFM_R,"----------")
  out <- rbind( out , tab )
  colnames(out ) <- c("Hyp","logLikHd","logLikHp","LR","log10LR")

  write.table(out,outf,sep="\t")

} #end loop for each hypothesis

} #end loop for each evidence
